/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMWindow;
interface nsIPrintSettings;
interface nsIWebProgressListener;

%{ C++
#include <stdio.h>
%}

[ptr] native FILE(FILE);

/**
 * The nsIDocShellFile    
 */

[scriptable, uuid(564a3276-6228-401e-9b5c-d82cb382a60f)]
interface nsIContentViewerFile : nsISupports
{
  readonly attribute boolean printable;

  [noscript] void print(in boolean aSilent,
                        in FILE    aDebugFile, 
                        in nsIPrintSettings aPrintSettings);
};
