/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef WordMovementType_h___
#define WordMovementType_h___

namespace mozilla {

enum EWordMovementType { eStartWord, eEndWord, eDefaultBehavior };

} // namespace mozilla

#endif
