#define MOZ_SOURCE_STAMP 2f2b4a119565e9b5691187ee5fbe91881c90b249
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/2f2b4a119565e9b5691187ee5fbe91881c90b249
