/* THIS FILE IS AUTOGENERATED FROM NotificationEvent.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_NotificationEventBinding_h
#define mozilla_dom_NotificationEventBinding_h

#include "ExtendableEventBinding.h"
#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Notification.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct NativePropertyHooks;
class Notification;
struct NotificationEventInitAtoms;
class ProtoAndIfaceCache;

namespace workers {

class NotificationEvent;
struct NotificationEventAtoms;

} // namespace workers

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct NotificationEventInit : public ExtendableEventInit
{
  MOZ_INIT_OUTSIDE_CTOR OwningNonNull<mozilla::dom::Notification> mNotification;

  NotificationEventInit();

  explicit inline NotificationEventInit(const FastDictionaryInitializer& )
    : ExtendableEventInit(FastDictionaryInitializer())
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

  explicit inline NotificationEventInit(const NotificationEventInit& aOther)
  {
    *this = aOther;
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const NotificationEventInit& aOther);

private:
  static bool
  InitIds(JSContext* cx, NotificationEventInitAtoms* atomsCache);
};

namespace binding_detail {
struct FastNotificationEventInit : public NotificationEventInit
{
  inline FastNotificationEventInit()
    : NotificationEventInit(FastDictionaryInitializer())
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


namespace NotificationEventBinding {

  typedef mozilla::dom::workers::NotificationEvent NativeType;

  bool
  ConstructorEnabled(JSContext* aCx, JS::Handle<JSObject*> aObj);

  const JSClass*
  GetJSClass();

  bool
  Wrap(JSContext* aCx, mozilla::dom::workers::NotificationEvent* aObject, nsWrapperCache* aCache, JS::Handle<JSObject*> aGivenProto, JS::MutableHandle<JSObject*> aReflector);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, T* aObject, JS::Handle<JSObject*> aGivenProto)
  {
    JS::Rooted<JSObject*> reflector(aCx);
    return Wrap(aCx, aObject, aObject, aGivenProto, &reflector) ? reflector.get() : nullptr;
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObjectHandle(JSContext* aCx);

  JS::Handle<JSObject*>
  GetConstructorObjectHandle(JSContext* aCx, bool aDefineOnGlobal = true);

  JSObject*
  GetConstructorObject(JSContext* aCx);

} // namespace NotificationEventBinding



} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_NotificationEventBinding_h
