/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsISystemStatusBar.idl
 */

#ifndef __gen_nsISystemStatusBar_h__
#define __gen_nsISystemStatusBar_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIDOMElement; /* forward declaration */


/* starting interface:    nsISystemStatusBar */
#define NS_ISYSTEMSTATUSBAR_IID_STR "24493180-ee81-4b7c-8b17-9e69480b7b8a"

#define NS_ISYSTEMSTATUSBAR_IID \
  {0x24493180, 0xee81, 0x4b7c, \
    { 0x8b, 0x17, 0x9e, 0x69, 0x48, 0x0b, 0x7b, 0x8a }}

class NS_NO_VTABLE nsISystemStatusBar : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_ISYSTEMSTATUSBAR_IID)

  /* void addItem (in nsIDOMElement aDOMMenuElement); */
  NS_IMETHOD AddItem(nsIDOMElement *aDOMMenuElement) = 0;

  /* void removeItem (in nsIDOMElement aDOMMenuElement); */
  NS_IMETHOD RemoveItem(nsIDOMElement *aDOMMenuElement) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsISystemStatusBar, NS_ISYSTEMSTATUSBAR_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSISYSTEMSTATUSBAR \
  NS_IMETHOD AddItem(nsIDOMElement *aDOMMenuElement) override; \
  NS_IMETHOD RemoveItem(nsIDOMElement *aDOMMenuElement) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSISYSTEMSTATUSBAR \
  NS_METHOD AddItem(nsIDOMElement *aDOMMenuElement); \
  NS_METHOD RemoveItem(nsIDOMElement *aDOMMenuElement); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSISYSTEMSTATUSBAR(_to) \
  NS_IMETHOD AddItem(nsIDOMElement *aDOMMenuElement) override { return _to AddItem(aDOMMenuElement); } \
  NS_IMETHOD RemoveItem(nsIDOMElement *aDOMMenuElement) override { return _to RemoveItem(aDOMMenuElement); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSISYSTEMSTATUSBAR(_to) \
  NS_IMETHOD AddItem(nsIDOMElement *aDOMMenuElement) override { return !_to ? NS_ERROR_NULL_POINTER : _to->AddItem(aDOMMenuElement); } \
  NS_IMETHOD RemoveItem(nsIDOMElement *aDOMMenuElement) override { return !_to ? NS_ERROR_NULL_POINTER : _to->RemoveItem(aDOMMenuElement); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsSystemStatusBar : public nsISystemStatusBar
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSISYSTEMSTATUSBAR

  nsSystemStatusBar();

private:
  ~nsSystemStatusBar();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsSystemStatusBar, nsISystemStatusBar)

nsSystemStatusBar::nsSystemStatusBar()
{
  /* member initializers and constructor code */
}

nsSystemStatusBar::~nsSystemStatusBar()
{
  /* destructor code */
}

/* void addItem (in nsIDOMElement aDOMMenuElement); */
NS_IMETHODIMP nsSystemStatusBar::AddItem(nsIDOMElement *aDOMMenuElement)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void removeItem (in nsIDOMElement aDOMMenuElement); */
NS_IMETHODIMP nsSystemStatusBar::RemoveItem(nsIDOMElement *aDOMMenuElement)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsISystemStatusBar_h__ */
